package com.huimv.env.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.entity.EnvWaterDay;
import com.huimv.env.common.mapper.EnvWaterDayMapper;
import com.huimv.env.common.service.EnvWaterDayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.common.utils.Const;
import com.huimv.env.common.utils.DateUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.Timestamp;
import java.text.ParseException;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-10-20
 */
@Service
public class EnvWaterDayServiceImpl extends ServiceImpl<EnvWaterDayMapper, EnvWaterDay> implements EnvWaterDayService {
    @Resource
    private EnvWaterDayMapper envWaterDayMapper;

    @Override
    public EnvWaterDay getDayWaterConsumption(String farmCode, String deviceCode, String sensorSn, Date todayDate) {
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.eq("add_date", todayDate);
        return envWaterDayMapper.selectOne(queryWrapper);
    }

    @Override
    public Result getTodayWater(Map<String, String> paramsMap) throws ParseException {
        DateUtil dateUtil = new DateUtil();
        String farmCode = paramsMap.get("farmCode");
        String deviceCode = paramsMap.get("deviceCode");
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
        queryWrapper.eq("add_date", dateUtil.getTodayDate());
        //读取当天的记录
        EnvWaterDay envWaterDay = envWaterDayMapper.selectOne(queryWrapper);
        if(envWaterDay == null){
            return new Result(Const.ERR_TODAY_WATER_CODE,Const.ERR_TODAY_WATER_INFO,false);
        }else{
            return new Result(ResultCode.SUCCESS,envWaterDay);
        }
    }

    @Override
    public void saveDayWater(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        String newWaterValue = dataJo.getString("value");
        BigDecimal newWaterBd = new BigDecimal(newWaterValue).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        //获取环比
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio(newWaterBd, deviceCode,  sensorSn,  todayDate,  farmCode);
        EnvWaterDay envWaterDay = new EnvWaterDay();
        envWaterDay.setDeviceCode(dataJo.getString("deviceCode"));
        envWaterDay.setStartTime(nowTimestamp);
        envWaterDay.setStartWater(newWaterBd);
        envWaterDay.setEndTime(nowTimestamp);
        envWaterDay.setEndWater(newWaterBd);
        envWaterDay.setDayWater(new BigDecimal(0));
        envWaterDay.setDataUnit("立方米");
        envWaterDay.setSensorSn(dataJo.getInteger("sensorSn"));
        envWaterDay.setAddDate(todayDate);
        envWaterDay.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envWaterDay.setUnitId(envDeviceRegisterEntity.getUnitId());
        envWaterDay.setLinkRelativeRatio(linkRelativeRatioBd);
        envWaterDay.setFarmCode(farmCode);
        int rows = envWaterDayMapper.insert(envWaterDay);
        System.out.println("添加记录数="+rows);
    }

    @Override
    public void updateDayWater(EnvWaterDay envWaterDay, JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, Date todayDate, String farmCode) {
        BigDecimal endWaterBd = new BigDecimal(dataJo.getString("value"));
        BigDecimal newDayWaterBd = (endWaterBd.subtract(envWaterDay.getStartWater())).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        //获取环比
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio(newDayWaterBd, deviceCode,  sensorSn,  todayDate,  farmCode);
        envWaterDay.setEndTime(nowTimestamp);
        envWaterDay.setEndWater(endWaterBd);
        envWaterDay.setDayWater(newDayWaterBd);
        envWaterDay.setLinkRelativeRatio(linkRelativeRatioBd);
        int rows = envWaterDayMapper.updateById(envWaterDay);
        System.out.println("编辑记录数="+rows);
    }

    //读取环比
    private BigDecimal _getLinkRelativeRatio(BigDecimal newWaterBd, String deviceCode, String sensorSn, Date todayDate, String farmCode) {
        QueryWrapper<EnvWaterDay> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.ne("add_date", todayDate);
        queryWrapper.orderByDesc("id");
        queryWrapper.last("LIMIT 1");
        EnvWaterDay lastEnvWaterDay = envWaterDayMapper.selectOne(queryWrapper);
        if(lastEnvWaterDay == null){
            return new BigDecimal(0);
        }else{
            return (newWaterBd.subtract(lastEnvWaterDay.getDayWater())).divide(lastEnvWaterDay.getDayWater()).multiply(new BigDecimal(100));
        }
    }
}
