package com.huimv.env.common.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.common.utils.Result;
import com.huimv.common.utils.ResultCode;
import com.huimv.env.common.dao.entity.EnvDeviceRegisterEntity;
import com.huimv.env.common.entity.EnvWaterMonth;
import com.huimv.env.common.mapper.EnvWaterMonthMapper;
import com.huimv.env.common.service.EnvWaterMonthService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author zn
 * @since 2022-10-20
 */
@Service
public class EnvWaterMonthServiceImpl extends ServiceImpl<EnvWaterMonthMapper, EnvWaterMonth> implements EnvWaterMonthService {
    @Resource
    private EnvWaterMonthMapper envWaterMonthMapper;

    @Override
    public EnvWaterMonth getMonthWaterConsumption(String farmCode, String deviceCode, String sensorSn, String year, String month) {
        QueryWrapper<EnvWaterMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.eq("year", year);
        queryWrapper.eq("month",month);
        return envWaterMonthMapper.selectOne(queryWrapper);
    }

    @Override
    public Result getMonthWater(Map<String, String> paramsMap) {
        String farmCode = paramsMap.get("farmCode");
        String months = paramsMap.get("months");
        QueryWrapper<EnvWaterMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.orderByDesc("year").orderByDesc("month");
        queryWrapper.last("LIMIT "+months);
        List<EnvWaterMonth> envWaterMonthList = envWaterMonthMapper.selectList(queryWrapper);
        List monthList = new ArrayList();
        for(int a=envWaterMonthList.size()-1;a>=0;a--){
            monthList.add(envWaterMonthList.get(a));
        }
        return new Result(ResultCode.SUCCESS,monthList);
    }

    @Override
    public void saveMonthWater(JSONObject dataJo, EnvDeviceRegisterEntity envDeviceRegisterEntity, Timestamp nowTimestamp, String year, String month, String farmCode) {
        String newWaterValue = dataJo.getString("value");
        BigDecimal newWaterBd = new BigDecimal(newWaterValue).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        //获取月份水量环比
        BigDecimal linkRelativeRatio = _getLinkRelativeRatio(newWaterBd,deviceCode,sensorSn,year,month,farmCode);
        EnvWaterMonth envWaterMonth = new EnvWaterMonth();
        envWaterMonth.setDeviceCode(deviceCode);
        envWaterMonth.setStartTime(nowTimestamp);
        envWaterMonth.setStartWater(newWaterBd);
        envWaterMonth.setEndTime(nowTimestamp);
        envWaterMonth.setEndWater(newWaterBd);
        envWaterMonth.setMonthWater(new BigDecimal(0));
        envWaterMonth.setDataUnit("立方米");
        envWaterMonth.setPigpenId(envDeviceRegisterEntity.getPigpenId());
        envWaterMonth.setUnitId(envDeviceRegisterEntity.getUnitId());
        envWaterMonth.setYear(Integer.parseInt(year));
        envWaterMonth.setMonth(Integer.parseInt(month));
        envWaterMonth.setSensorSn(dataJo.getInteger("sensorSn"));
        envWaterMonth.setLinkRelativeRatio(linkRelativeRatio);
        envWaterMonth.setFarmCode(farmCode);
        int rows = envWaterMonthMapper.insert(envWaterMonth);
        System.out.println("添加记录数="+rows);
    }

    //读取环比
    private BigDecimal _getLinkRelativeRatio(BigDecimal newWaterBd, String deviceCode, String sensorSn, String year, String month, String farmCode) {
        QueryWrapper<EnvWaterMonth> queryWrapper = new QueryWrapper();
        queryWrapper.eq("farm_code", farmCode);
        queryWrapper.eq("device_code", deviceCode);
//        queryWrapper.eq("sensor_sn",sensorSn);
        queryWrapper.and(wrapper->wrapper.ne("year",year).or().ne("month",month));
        queryWrapper.orderByDesc("id");
        queryWrapper.last("LIMIT 1");
        EnvWaterMonth lastEnvWaterMonth = envWaterMonthMapper.selectOne(queryWrapper);
        if(lastEnvWaterMonth == null){
            return new BigDecimal(0);
        }else{
            return (newWaterBd.subtract(lastEnvWaterMonth.getMonthWater())).divide(lastEnvWaterMonth.getMonthWater()).multiply(new BigDecimal(100));
        }
    }

    @Override
    public void updateMonthWater(EnvWaterMonth envWaterMonth, JSONObject dataJo, Timestamp nowTimestamp, String year, String month, String farmCode) {
        BigDecimal endWaterBd = new BigDecimal(dataJo.getString("value"));
        BigDecimal newMonthWaterBd = (endWaterBd.subtract(envWaterMonth.getStartWater())).divide(new BigDecimal("100"), 2, BigDecimal.ROUND_HALF_UP);
        String deviceCode = dataJo.getString("deviceCode");
        String sensorSn = dataJo.getString("sensorSn");
        BigDecimal linkRelativeRatioBd = _getLinkRelativeRatio(newMonthWaterBd,deviceCode,sensorSn,year,month,farmCode);
        envWaterMonth.setEndTime(nowTimestamp);
        envWaterMonth.setEndWater(endWaterBd);
        envWaterMonth.setMonthWater(newMonthWaterBd);
        envWaterMonth.setLinkRelativeRatio(linkRelativeRatioBd);
        int rows = envWaterMonthMapper.updateById(envWaterMonth);
        System.out.println("编辑记录数="+rows);
    }
}
