package com.huimv.env.manage.saas.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.huimv.env.manage.entity.EnvDeviceRegister;
import com.huimv.env.manage.mapper.BasePigpenMapper;
import com.huimv.env.manage.mapper.LampConfigAllMapper;
import com.huimv.env.manage.saas.dao.entity.BasePigpen;
import com.huimv.env.manage.saas.dao.entity.LampConfig;
import com.huimv.env.manage.mapper.LampConfigMapper;
import com.huimv.env.manage.saas.dao.entity.LampConfigAll;
import com.huimv.env.manage.saas.service.IBasePigpenService;
import com.huimv.env.manage.saas.service.ILampConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.env.manage.utils.Result;
import com.huimv.env.manage.utils.ResultCode;
import com.huimv.env.manage.utils.ResultUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2022-10-18
 */
@Service
public class LampConfigServiceImpl extends ServiceImpl<LampConfigMapper, LampConfig> implements ILampConfigService {

    @Autowired
    private LampConfigMapper lampConfigMapper;
    @Autowired
    private LampConfigAllMapper lampConfigAllMapper;

    @Autowired
    BasePigpenMapper basePigpenMapper;

    @Override
    public Result savelampConfig( LampConfig lampConfig, HttpServletRequest request) {
        String deviceId = lampConfig.getDeviceId();
        Integer unitId = lampConfig.getUnitId();
        String location  = lampConfig.getLocation();
        if (lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("device_id",deviceId))){
            return new Result(10001,"保温灯已存在",false);
        }
        if (lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("unit_id",unitId).eq("location",location))){
            return new Result(10002,"该位置已有保温灯",false);
        }
        this.save(lampConfig);
        return new Result(10000,"添加成功",true);

    }

    @Override
    public Result lsitAllLamp(Map<String, String> map, HttpServletRequest request) {
        String farmCode = map.get("farmCode");
        List<LampConfig> lampConfigs = lampConfigMapper.selectList(new QueryWrapper<LampConfig>().eq("farm_code", farmCode));
        List endList = new ArrayList();
        for (LampConfig lampConfig : lampConfigs) {
            Map endMap = new HashMap();
            endMap.put("binding",lampConfig.getBinding());
            endMap.put("deviceId",lampConfig.getDeviceId());
            endMap.put("farmCode",lampConfig.getFarmCode());
            endMap.put("id",lampConfig.getId());
            endMap.put("unitId",lampConfig.getUnitId());
            endMap.put("lampName",lampConfig.getLampName());
            endMap.put("location",lampConfig.getLocation());
            BasePigpen basePigpen = basePigpenMapper.selectById(lampConfig.getUnitId());
            if (ObjectUtil.isNotEmpty(basePigpen)){
                endMap.put("pigpenName",basePigpen.getPigpenName());
            }else {
                endMap.put("pigpenName","该设备未绑定");
            }
            endList.add(endMap);
        }

        return ResultUtil.isNull(endList);
    }

    @Override
    public Result unbindingLamp(Map<String, String> map, HttpServletRequest request) {
        String deviceId = map.get("deviceId");
        LampConfig lampConfig = lampConfigMapper.selectOne(new QueryWrapper<LampConfig>().eq("device_id", deviceId));
        lampConfig.setLocation("");
        lampConfig.setBinding(0);
        lampConfig.setUnitId(0);
        lampConfig.setUploadStart(0);
        lampConfigMapper.updateById(lampConfig);
        return new Result(10000,"解绑成功",true);
    }

    @Override
    public Result bindingBaseLamp(Map<String, String> map, HttpServletRequest request) {
        String id = map.get("id");
        String unitId = map.get("unitId");
        String location = map.get("location");

        if (lampConfigMapper.exists(new QueryWrapper<LampConfig>().eq("unit_id",unitId).eq("location",location))){
            return new Result(10002,"该位置已有保温灯",false);
        }

        LampConfig lampConfig = new LampConfig();

        lampConfig.setId(Integer.parseInt(id));
        lampConfig.setUnitId(Integer.parseInt(unitId));
        lampConfig.setLocation(location);
        lampConfig.setBinding(1);

        lampConfigMapper.updateById(lampConfig);

        return new Result(10000,"绑定成功",true);
    }

    //根据日龄计算温度
    @Override
    public String updateTemp(String ageList, Integer dayAge) {

        String[] split = ageList.split(";");
        String temp = split[split.length-1].split(",")[1];
        for (int i = 0; i < split.length; i++) {
            String  lamp1 = split[i];
            if (dayAge<=Integer.parseInt(lamp1.split(",")[0])){
                temp = lamp1.split(",")[1];
                String tempStr = (Double.parseDouble(temp) * 10) + "";
                return  tempStr.substring(0,tempStr.lastIndexOf("."));
            }
        }

        String tempStr = (Double.parseDouble(temp) * 10) + "";

        return  tempStr.substring(0,tempStr.lastIndexOf("."));
    }

    @Override
    public Result allToConfig(Map<String, String> map, HttpServletRequest request) {
        try {
            String initDayAge =  map.get("initDayAge");
            String dayAgeList =  map.get("dayAgeList");
            //
            String boardGear = map.get("boardGear");
            String unitId = map.get("unitId");
            String id = map.get("id");

            JSONArray jsonArray = JSONUtil.parseArray(dayAgeList);

            LampConfig lampConfig = new LampConfig();
            lampConfig.setInitAge(Integer.parseInt(initDayAge));
            lampConfig.setBoardGear(Integer.parseInt(boardGear));
            StringBuilder dayAgeListStr = new StringBuilder();
            for (Object o : jsonArray) {
                dayAgeListStr.append(o.toString()).append(";");
            }
            lampConfig.setAgeList(dayAgeListStr.toString());
            UpdateWrapper<LampConfig> wrapper = new UpdateWrapper<>();
            if (StringUtils.isBlank(id)){
                LampConfigAll lampConfigAll = new LampConfigAll();
                lampConfigAll.setBoardGear(Integer.parseInt(boardGear));
//                lampConfigAll.setListDayage();
                lampConfigAll.setInitDayage(Integer.parseInt(initDayAge));
//                lampConfigAll.setFarmCode();
//                lampConfigAll.setUnitId();
                wrapper.eq("unit_id",unitId);
            }else {
                wrapper.eq("id",id);
            }
            lampConfigMapper.update(lampConfig,wrapper);

            return new Result(10000,"全局配置成功",true);
        }catch (Exception e){
            System.out.println(e);
            return new Result(10001,"配置失败",true);
        }

    }

    @Override
    public Result listByUnit(Map<String, String> map, HttpServletRequest request) {
        String unitId = map.get("unitId");
        QueryWrapper<LampConfig> wrapper = new QueryWrapper<>();
        wrapper.eq("unit_id",unitId);
        List<LampConfig> lampConfigs = lampConfigMapper.selectList(wrapper);
        return ResultUtil.isNull(lampConfigs);
    }

    @Override
    public Result countByUnit(Map<String, String> map, HttpServletRequest request) {
        String unitId = map.get("unitId");
        QueryWrapper<LampConfig> wrapper = new QueryWrapper<>();
        wrapper.eq("unit_id",unitId);
        Long countAll = lampConfigMapper.selectCount(wrapper);
        wrapper.eq("open_start",1);
        Long countStart = lampConfigMapper.selectCount(wrapper);
        Long countClose =countAll -countStart;
        Map endMap = new HashMap();
        endMap.put("countAll",countAll);
        endMap.put("countStart",countStart);
        endMap.put("countClose",countClose);
        if (countStart >0){
            endMap.put("start",true);
        }else {
            endMap.put("start",false);
        }

        return new Result(ResultCode.SUCCESS,endMap) ;
    }

    @Override
    public Result updateLamp(Map<String, Integer> map, HttpServletRequest request) {
        Integer uploadStart = map.get("uploadStart");
        Integer id = map.get("id");
        Integer unitId = map.get("unitId");
        LampConfig lampConfig = new LampConfig();
        lampConfig.setUploadStart(uploadStart);
        if (ObjectUtil.isNotEmpty(unitId)){
            lampConfigMapper.update(lampConfig,new QueryWrapper<LampConfig>().eq("unit_id",unitId));
        }else {
            lampConfig.setId(id);
            lampConfigMapper.updateById(lampConfig);
        }

        return new Result(10000,"配置成功",true);
    }

    @Override
    public Result listByLampId(Map<String, String> map, HttpServletRequest request) {
        String lampId = map.get("lampId");
        LampConfig lampConfig = lampConfigMapper.selectById(lampId);
        if (ObjectUtil.isEmpty(lampConfig)){
            lampConfig = new LampConfig();
            lampConfig.setInitAge(0);
            lampConfig.setAgeList("0,33;");
        }
        Map endMap =new HashMap();
        endMap.put("initDayage",lampConfig.getInitAge());
        endMap.put("listDayage",lampConfig.getAgeList().split(";"));
        endMap.put("id",lampConfig.getId());
        endMap.put("lampName",lampConfig.getLampName());
        endMap.put("deviceId",lampConfig.getDeviceId());
        return new Result(ResultCode.SUCCESS,endMap);
    }
}
